@@ -1,14 +1,13 @@
-# 2022.7.4 merge verbnet and terms code 
-# 2022.5.28, redisgears's python is 3.7 , to be compatible 
-# 2022.3.20, usage:  import en |  spacy.nlp("hello")   
+# 2022.7.4 merge verbnet and terms code  | # 2022.5.28, redisgears's python is 3.7 , to be compatible | # 2022.3.20, usage:  import en |  spacy.nlp("hello")   
 import json,spacy,os,builtins,time,fileinput, sqlite3,traceback,sys
 from spacy.tokens import DocBin,Doc,Token
 from spacy.tokenizer import Tokenizer
+from spacy.language import Language
 from spacy.matcher import Matcher,DependencyMatcher
-from spacy.lang.char_classes import ALPHA, ALPHA_LOWER, ALPHA_UPPER, CONCAT_QUOTES, LIST_ELLIPSES, LIST_ICONS
-from spacy.util import compile_infix_regex
 
 def custom_tokenizer(nlp): #https://stackoverflow.com/questions/58105967/spacy-tokenization-of-hyphenated-words
+	from spacy.lang.char_classes import ALPHA, ALPHA_LOWER, ALPHA_UPPER, CONCAT_QUOTES, LIST_ELLIPSES, LIST_ICONS
+	from spacy.util import compile_infix_regex
 	infixes = (
 		LIST_ELLIPSES
 		+ LIST_ICONS
@@ -25,18 +24,10 @@ def custom_tokenizer(nlp): #https://stackoverflow.com/questions/58105967/spacy-t
 	infix_re = compile_infix_regex(infixes)
 	return Tokenizer(nlp.vocab, prefix_search=nlp.tokenizer.prefix_search,suffix_search=nlp.tokenizer.suffix_search,infix_finditer=infix_re.finditer,token_match=nlp.tokenizer.token_match,	rules=nlp.Defaults.tokenizer_exceptions)
 
-def to_docbin(doc):
-	doc_bin= spacy.tokens.DocBin()
-	doc_bin.add(doc)
-	return doc_bin.to_bytes()
-
-def from_docbin(bs): 
-	return list(spacy.tokens.DocBin().from_bytes(bs).get_docs(spacy.nlp.vocab))[0] if bs else None
-
 if not hasattr(spacy, 'nlp'):
 	spacy.nlp		= spacy.load(os.getenv('spacy_model','en_core_web_lg')) # 3.4.1
-	spacy.tobs		= to_docbin # for old codes
-	spacy.frombs	= from_docbin
+	#spacy.tobs		= to_docbin # for old codes
+	#spacy.frombs	= from_docbin
 	spacy.from_json = lambda arr: Doc(spacy.nlp.vocab).from_json(arr) # added 2022.8.19
 	spacy.nlp.tokenizer = custom_tokenizer(spacy.nlp)	#nlp.tokenizer.infix_finditer = infix_re.finditer
 	#print([t.text for t in nlp("It's 1.50, up-scaled haven't")]) # ['It', "'s", "'", '1.50', "'", ',', 'up-scaled', 'have', "n't"]
@@ -44,21 +35,10 @@ if not hasattr(spacy, 'nlp'):
 def refresh(): # to release spacy's memory , added 2023.1.20
 	spacy.nlp = spacy.load(os.getenv('spacy_model','en_core_web_lg'))
 
-def model(name:str='sm'):
-	''' sm/md/lg, doc = en.model('lg')("hello world"),  2023.1.17 '''
-	if not hasattr(model, name):  
-		setattr(model, name, spacy.load(f'en_core_web_{name}') )
-	return getattr(model, name) 
-lookup	= lambda s, name='lg': model(name).vocab[s].text
-fromjson= lambda jsons, name='lg': Doc(model(name).vocab).from_json(jsons)
-
-def getdoc(snt, bs):  # execute("GET", f"bytes:{snt}") 
-	return Doc(spacy.nlp.vocab).from_bytes(bs) if bs else spacy.nlp(snt) 
-
-def phrase_matcher(name:str='pp', patterns:list=[[{'POS': 'ADP'},{"POS": {"IN": ["DET","NUM","ADJ",'PUNCT','CONJ']}, "OP": "*"},{"POS": {"IN": ["NOUN","PART"]}, "OP": "+"}]]):
+def phrase_matcher( rules ={'pp':[[{'POS': 'ADP'},{"POS": {"IN": ["DET","NUM","ADJ",'PUNCT','CONJ']}, "OP": "*"},{"POS": {"IN": ["NOUN","PART"]}, "OP": "+"}]] }):
 	''' for name, ibeg,iend in matcher(doc) : print(spacy.nlp.vocab[name].text, doc[ibeg:iend].text) '''
 	matcher = Matcher(spacy.nlp.vocab)
-	matcher.add(name, patterns,  greedy ='LONGEST')
+	[matcher.add(name, pats,  greedy ='LONGEST') for name, pats in rules.items()]
 	return matcher
 
 def sntbr(essay, trim:bool=False, with_pid:bool=False): 
@@ -78,26 +58,20 @@ def sntbr(essay, trim:bool=False, with_pid:bool=False):
 		arr.append( (snt.text, pid) ) 
 	return arr 
 
-#token_split	= lambda sent: re.findall(r"[\w']+|[.,!?;]", sent) # return list
 def common_perc(snt="She has ready.", trans="She is ready."): 
 	toks = set([t.text for t in spacy.nlp.tokenizer(snt)])
 	return len([t for t in spacy.nlp.tokenizer(trans) if t.text in toks]) / (len(toks)+0.01)
 
-merge_nps		= lambda : spacy.nlp.create_pipe("merge_noun_chunks")
+merge_nps		= spacy.nlp.create_pipe("merge_noun_chunks") #merge_entities
+merge_ent		= spacy.nlp.create_pipe("merge_entities") 
 new_matcher		= lambda : Matcher(spacy.nlp.vocab) # by exchunk
 toks			= lambda doc:  [{'i':t.i, "head":t.head.i, 'lex':t.text, 'lem':t.lemma_, 'pos':t.pos_, 'tag':t.tag_, 'dep':t.dep_, "gpos":t.head.pos_, "glem":t.head.lemma_} for t in doc ] # JSONEachRow 
 postag			= lambda doc:  "_^ " + " ".join([ f"{t.text}_{t.lemma_}_{t.pos_}_{t.tag_}" for t in doc]) + " _$"
 non_root_verbs	= lambda doc:  [t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT'] 
 simple_sent		= lambda doc:  len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) <= 0 # else is complex sent 
 compound_snt	= lambda doc:  len([t for t in doc if t.dep_ == 'conj' and t.head.dep_ == 'ROOT']) > 0
-snt_source		= lambda sid, doc: {'type':'snt', 'src': sid, 'snt':doc.text, 'pred_offset': pred_offset(doc), 
-				'postag':'_^ ' + ' '.join([f"{t.text}_{t.lemma_}_{t.pos_}_{t.tag_}" if t.text == t.text.lower() else f"{t.text}_{t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}" for t in doc]) + ' _$',
-			   'tc': len(doc)}
-
-def pred_offset(doc):
-	ar = [ t.i for t in doc if t.dep_ == "ROOT"]
-	offset = ar[0] if len(ar) > 0 else 0
-	return offset/( len(doc) + 0.1) 
+snt_source		= lambda sid, doc: {'type':'snt', 'src': sid, 'snt':doc.text, 'pred_offset': pred_offset(doc), 	'postag':'_^ ' + ' '.join([f"{t.text}_{t.lemma_}_{t.pos_}_{t.tag_}" if t.text == t.text.lower() else f"{t.text}_{t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}" for t in doc]) + ' _$',  'tc': len(doc)}
+pred_offset		= lambda doc: ( ar := [ t.i for t in doc if t.dep_ == "ROOT"], offset := ar[0] if len(ar) > 0 else 0, offset/( len(doc) + 0.1) )[-1]
 
 def show(doc):
 	''' used in the notebook, for debug '''
@@ -105,8 +79,8 @@ def show(doc):
 	return pd.DataFrame({'word': [t.text for t in doc], 'tag': [t.tag_ for t in doc],'pos': [t.pos_ for t in doc],'head': [t.head.orth_ for t in doc],'dep': [t.dep_ for t in doc], 'lemma': [t.text.lower() if t.lemma_ == '-PRON-' else t.lemma_ for t in doc],
 	'n_lefts': [ t.n_lefts for t in doc], 'left_edge': [ t.left_edge.text for t in doc], 
 	'n_rights': [ t.n_rights for t in doc], 'right_edge': [ t.right_edge.text for t in doc],
-	'subtree': str([ list(t.subtree) for t in doc]),'children': str([ list(t.children) for t in doc]),
-	'morph': [ t.morph for t in doc],'ent_type': [ t.ent_type_ for t in doc],
+	'subtree': [ list(t.subtree) for t in doc], 'children': [ list(t.children) for t in doc], 	#'morph': [ t.morph for t in doc],
+	'ent_type': [ t.ent_type_ for t in doc], 'ent_id': [ t.ent_id_ for t in doc],
 	})
 
 def parse(snt, merge_np= False):
@@ -163,7 +137,7 @@ def kp_matcher(doc): #[('vend', 'consider going', 1, 3), ('vp', 'consider going'
 	return tups
 
 def ruler(snt, name:str='sm'):
-	''' 2023.1.21 '''
+	''' doc.spans['ruler'] 2023.1.21 '''
 	if not hasattr(ruler, name):  
 		_nlp = spacy.load(f'en_core_web_{name}')
 		setattr(ruler, name, _nlp )
@@ -214,6 +188,7 @@ matchers = {  # for name,start,end in matchers['ap'](doc) :
 ##
 ## from verbnet.py
 ##
+@Language.component("merge_np")
 def merge_np(doc):
 	with doc.retokenize() as retokenizer:
 		for np in doc.noun_chunks:
@@ -240,6 +215,7 @@ def merge_n_of_n(doc):
 					print ( "merge_n_of_n ex:", e , start, end)
 	return doc
 
+@Language.component("merge_vp")
 def merge_vp(doc):
 	if not hasattr(merge_vp, 'matcher'):
 		merge_vp.matcher = Matcher(spacy.nlp.vocab)
@@ -287,6 +263,21 @@ def merge_clause(doc): # subtree of a verb is the clause , https://subscription.
 				print ( "merge_clause ex:", e, v )
 	return doc
 
+@Language.component("span_clause")
+def span_clause(doc): # {'cl:4': [that it cried], 'cl:10': [that I would fail]}
+	from spacy.tokens import SpanGroup
+	for v in [t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT' ] : # non-root
+		try:
+			children = list(v.subtree)
+			start = children[0].i  	
+			end = children[-1].i 
+			attrs = {"pos": v.pos_, "tag": v.tag_, "dep": v.dep_, "lemma":v.lemma_, "ent_type": "S." + v.dep_ } # S.advcl ,  S.conj 
+			name = f"cl:{start}" 
+			doc.spans[name] = SpanGroup(doc, name=name, spans=[ doc[start : end+1] ], attrs=attrs)
+		except Exception as e:
+			print ( "span_clause ex:", e, v )
+	return doc
+
 NP_start = {"ENT_TYPE": "NP", "IS_SENT_START": True}
 VERB	 = {"POS": {"IN": ["VERB"]}}
 NOUN	 = {"POS": {"IN": ["NOUN","PRON","PROPN"]}}
@@ -363,228 +354,27 @@ def vp_matcher(doc): #[('vend', 'consider going', 1, 3), ('vp', 'consider going'
 
 _dep_rules = {
 "svo":[ {"RIGHT_ID": "v",  "RIGHT_ATTRS": {"POS": "VERB"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"}},{"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object","RIGHT_ATTRS": {"DEP": "dobj"}  }], # [(4851363122962674176, [2, 0, 4])]
-"sva":[ 
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "subject",
-    "RIGHT_ATTRS": {"DEP": "nsubj"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "object",
-    "RIGHT_ATTRS": {"DEP": "acomp"}
-  }
-], 
-"svx":[  # plan to go , enjoy swimming 
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "subject",
-    "RIGHT_ATTRS": {"DEP": "nsubj"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "object",
-    "RIGHT_ATTRS": {"DEP": "xcomp"}
-  }
-], 
-"svc":[  # I think it is right.
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "subject",
-    "RIGHT_ATTRS": {"DEP": "nsubj"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "object",
-    "RIGHT_ATTRS": {"DEP": "ccomp"}
-  }
-], 
-"sattr":[  #She is  a girl.
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"LEMMA": "be"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "subject",
-    "RIGHT_ATTRS": {"DEP": "nsubj"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "object",
-    "RIGHT_ATTRS": {"DEP": "attr"}
-  }
-], 
-"vpn":[ # turn off the light
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "subject",
-    "RIGHT_ATTRS": {"DEP": "prt"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "object",
-    "RIGHT_ATTRS": {"DEP": "dobj"}
-  }
-], 
-"vap":[ # be happy with
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "acomp",
-    "RIGHT_ATTRS": {"DEP": "acomp"}
-  },
-  {
-    "LEFT_ID": "acomp",
-    "REL_OP": ">",
-    "RIGHT_ID": "prep",
-    "RIGHT_ATTRS": {"DEP": "prep"}
-  }
-], 
-"vdp":[ # be based on
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"TAG": "VBN"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "be",
-    "RIGHT_ATTRS": {"LEMMA": "be"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "prep",
-    "RIGHT_ATTRS": {"DEP": "prep"}
-  }
-], 
-"vppn":[ # look up from phone
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "prt",
-    "RIGHT_ATTRS": {"DEP": "prt"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "prep",
-    "RIGHT_ATTRS": {"DEP": "prep"}
-  },
-  {
-    "LEFT_ID": "prep",
-    "REL_OP": ">",
-    "RIGHT_ID": "object",
-    "RIGHT_ATTRS": {"DEP": "pobj"}
-  }
-], 
-"vpnpn":[ # vary from A to B
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "prep1",
-    "RIGHT_ATTRS": {"DEP": "prep"}
-  },
-  {
-    "LEFT_ID": "prep1",
-    "REL_OP": ">",
-    "RIGHT_ID": "object1",
-    "RIGHT_ATTRS": {"DEP": "pobj"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "prep2",
-    "RIGHT_ATTRS": {"DEP": "prep"}
-  },
-  {
-    "LEFT_ID": "prep2",
-    "REL_OP": ">",
-    "RIGHT_ID": "object2",
-    "RIGHT_ATTRS": {"DEP": "pobj"}
-  }
-], 
-"vnp":[ # turn it down
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "object",
-    "RIGHT_ATTRS": {"DEP": "dobj"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "subject",
-    "RIGHT_ATTRS": {"DEP": "prt"}
-  }
-], 
-"vnpn":[  # make use of books, take sth into account
-  {
-    "RIGHT_ID": "v",
-    "RIGHT_ATTRS": {"POS": "VERB"}
-  },
-  {
-    "LEFT_ID": "v",
-    "REL_OP": ">",
-    "RIGHT_ID": "object",
-    "RIGHT_ATTRS": {"DEP": "dobj"}
-  },
-  {
-    "LEFT_ID": "object",
-    "REL_OP": ">",
-    "RIGHT_ID": "prep",
-    "RIGHT_ATTRS": {"DEP": "prep"}
-  },
-  {
-    "LEFT_ID": "prep",
-    "REL_OP": ">",
-    "RIGHT_ID": "pobj",
-    "RIGHT_ATTRS": {"DEP": "pobj"}
-  }  
-], 
+"sva":[ {"RIGHT_ID": "v",  "RIGHT_ATTRS": {"POS": "VERB"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"}},{"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object","RIGHT_ATTRS": {"DEP": "acomp"}}], 
+# plan to go , enjoy swimming 
+"svx":[ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object","RIGHT_ATTRS": {"DEP": "xcomp"}}], 
+# I think it is right.
+"svc":[ {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "nsubj"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "ccomp"}}], 
+#She is  a girl.
+"sattr":[ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"LEMMA": "be"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject","RIGHT_ATTRS": {"DEP": "nsubj"}},{"LEFT_ID": "v","REL_OP": ">", "RIGHT_ID": "object","RIGHT_ATTRS": {"DEP": "attr"}  }], 
+# turn off the light
+"vpn":[ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "prt"}}, {"LEFT_ID": "v", "REL_OP": ">","RIGHT_ID": "object","RIGHT_ATTRS": {"DEP": "dobj"} }], 
+# be happy with
+"vap":[ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "acomp","RIGHT_ATTRS": {"DEP": "acomp"}},{"LEFT_ID": "acomp", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "prep"} }], 
+# be based on
+"vdp":[ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"TAG": "VBN"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "be", "RIGHT_ATTRS": {"LEMMA": "be"}},{"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "prep"}}], 
+# look up from phone
+"vppn":[ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}}, { "LEFT_ID": "v","REL_OP": ">", "RIGHT_ID": "prt", "RIGHT_ATTRS": {"DEP": "prt"}},{"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "prep"}}, {"LEFT_ID": "prep", "REL_OP": ">", "RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "pobj"}}], 
+# vary from A to B
+"vpnpn":[ {"RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}}, {"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "prep1", "RIGHT_ATTRS": {"DEP": "prep"}},{"LEFT_ID": "prep1", "REL_OP": ">", "RIGHT_ID": "object1", "RIGHT_ATTRS": {"DEP": "pobj"}},{"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "prep2", "RIGHT_ATTRS": {"DEP": "prep"}},{"LEFT_ID": "prep2", "REL_OP": ">", "RIGHT_ID": "object2", "RIGHT_ATTRS": {"DEP": "pobj"}}], 
+# turn it down
+"vnp":[ { "RIGHT_ID": "v","RIGHT_ATTRS": {"POS": "VERB"}},{"LEFT_ID": "v","REL_OP": ">","RIGHT_ID": "object", "RIGHT_ATTRS": {"DEP": "dobj"}},{"LEFT_ID": "v", "REL_OP": ">","RIGHT_ID": "subject", "RIGHT_ATTRS": {"DEP": "prt"}}], 
+# make use of books, take sth into account
+"vnpn":[ {"RIGHT_ID": "v", "RIGHT_ATTRS": {"POS": "VERB"}},{"LEFT_ID": "v", "REL_OP": ">", "RIGHT_ID": "object","RIGHT_ATTRS": {"DEP": "dobj"}},{"LEFT_ID": "object","REL_OP": ">", "RIGHT_ID": "prep", "RIGHT_ATTRS": {"DEP": "prep"}},{"LEFT_ID": "prep", "REL_OP": ">", "RIGHT_ID": "pobj", "RIGHT_ATTRS": {"DEP": "pobj"}}  ], 
 } # for name, ar in depmatchers['svx'](doc) : print(doc[ar[1]], doc[ar[0]], doc[ar[2]])
 
 def dep_matcher(doc): #[('svx', [1, 0, 2])]
@@ -633,12 +423,32 @@ def merge_ccomp(doc): #doc = spacy.nlp("While I was thrilled that it was ok, I w
 				print ( "merge_ccomp ex:", e, v )
 	return doc 
 
-def es_skenp(doc):
-	from dic.oneself import oneself 
-	merge_ccomp(doc) # merge_leave_cl(doc) 
-	merge_np(doc) # transform doc , finally to be called | f"_{t.lemma_}_{t.pos_}_{t.tag_}_cl_{t.dep_}"
-	return "_^ " + ' '.join([ f"_{t.lemma_}_NP_{t.dep_}" if t.ent_type_ == 'NP' else f"_CL_{t.dep_}" if t.ent_type_.startswith("S.") else f"_{t.pos_}_{t.tag_}_{t.dep_}" if t.pos_ in ('PROPN','NUM','X','SPACE') else f"{t.text if t.text in ('I') else t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}{oneself.get(t.text.lower(), '')}_{t.dep_}"  for t in doc]) # uniq by ana #,'PUNCT'
+def attach_cl(doc): #2023.1.23
+	for v in [t for t in doc if t.pos_ == 'VERB'] : # non-root
+		try:
+			children = list(v.subtree)
+			start = children[0].i  	
+			end = children[-1].i 
+			cl_len = end - start + 1 
+			doc[start].ent_id_ = f'_CL{cl_len}'  # add cl_root ? 
+		except Exception as e:
+			print ( "attach_cl ex:", e, v )
+	return doc 
 
+def skenp_tok(t): 
+	from dic.oneself import oneself 
+	pair = f"_{t.lemma_}_NP_{t.dep_}" if t.ent_type_ == 'NP' else f"_{t.pos_}_{t.tag_}_{t.dep_}" if t.pos_ in ('PROPN','NUM','X','SPACE') else f"{t.text if t.text in ('I') else t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}{oneself.get(t.text.lower(), '')}_{t.dep_}"
+	if t.ent_id_.startswith("_CL"): pair = pair + t.ent_id_
+	return pair 
+
+def es_skenp(doc):
+	#merge_ccomp(doc) # merge_leave_cl(doc) 
+	merge_np(doc) 
+	attach_cl(doc)
+	#return "_^ " + ' '.join([ f"_{t.lemma_}_NP_{t.dep_}" if t.ent_type_ == 'NP' else f"_CL_{t.dep_}" if t.ent_type_.startswith("S.") else f"_{t.pos_}_{t.tag_}_{t.dep_}" if t.pos_ in ('PROPN','NUM','X','SPACE') else f"{t.text if t.text in ('I') else t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}{oneself.get(t.text.lower(), '')}_{t.dep_}"  for t in doc]) 
+	return "_^ " + ' '.join([ skenp_tok(t) for t in doc]) 
+
+@Language.component("merge_prt")
 def merge_prt(doc): 
 	'''I turn off the radio. => turn_off , added 2023.1.13'''
 	if not hasattr(merge_prt, 'matcher'):
@@ -653,19 +463,69 @@ def merge_prt(doc):
 				print ( "merge_prt ex:", e , start, end)
 	return doc
 
-def merged(doc): 
-	if not hasattr(merged, 'matcher'):
-		merged.matcher = Matcher(spacy.nlp.vocab)
-		merged.matcher.add("vprt", [[{"POS":"VERB"}, {"POS":"ADP", "DEP":"prt"}]], greedy ='LONGEST')
-		merged.matcher.add("vtov", [[{"POS":"VERB"},{"LEMMA":"to", "TAG":"TO"}, {"TAG":"VB"}]], greedy ='LONGEST')        
-		merged.matcher.add("vvbg", [[{"POS":"VERB"},{"DEP":"xcomp","TAG":"VBG"}]], greedy ='LONGEST')                
+@Language.component("merge_v")
+def merge_v(doc): # 2023.1.22
+	if not hasattr(merge_v, 'matcher'):
+		merge_v.matcher = Matcher(spacy.nlp.vocab)
+		merge_v.matcher.add("vprt", [[{"POS":"VERB"}, {"POS":"ADP", "DEP":"prt"}]], greedy ='LONGEST')
+		#merge_v.matcher.add("vtov", [[{"POS":"VERB"},{"LEMMA":"to", "TAG":"TO"}, {"TAG":"VB"}]], greedy ='LONGEST')      # plan to turn off the radio 
+		#merge_v.matcher.add("vvbg", [[{"POS":"VERB"},{"DEP":"xcomp","TAG":"VBG"}]], greedy ='LONGEST')                
+		merge_v.matcher.add("be-pn", [[{"LEMMA":"be"},{"TAG":"IN","POS":"ADP"},{"TAG":"NN"}]], greedy ='LONGEST') # be in construction
+		merge_v.matcher.add("be-acomp", [[{"LEMMA":"be"},{"DEP":"acomp","POS":"ADJ"}]], greedy ='LONGEST') #"POS":"AUX",
+		merge_v.matcher.add("be-auxpass", [[{"LEMMA":"be","DEP":"auxpass"},{"TAG":"VBN","POS":"VERB"}]], greedy ='LONGEST')
 	with doc.retokenize() as retokenizer:
-		for name, start, end in merged.matcher(doc):
-			try:
-				attrs = {"pos": doc[start].pos, "tag": doc[start].tag, "dep": doc[start].dep, "lemma":"_".join([doc[i].lemma_ if i == start else doc[i].text for i in range(start,end)]), "ent_type": "merged_" + spacy.nlp.vocab[name].text}
+		for name, start, end in merge_v.matcher(doc):
+			try: 
+				attrs = {"pos": "VERB", "tag": doc[start].tag, "dep": doc[start].dep, 
+					"lemma":"_".join([doc[i].lemma_ if i == start else doc[i].text for i in range(start,end)]), 
+					"ent_type": "merge_v:" + spacy.nlp.vocab[name].text + f"|{doc[start].lemma_}:{doc[start].pos_}:{doc[start+1].dep_}:{doc[start+1].pos_}:{doc[start+1].lemma_}" }
 				retokenizer.merge(doc[start : end], attrs=attrs)
 			except Exception as e:
-				print ( "merged ex:", e , start, end)
+				print ( "merge_v ex:", e , start, end)
+	return doc
+
+@Language.component("merge_trp")
+def merge_trp(doc): # 2023.1.24, |two words, the second one is the head 
+	if not hasattr(merge_trp, 'matcher'):
+		merge_trp.matcher = Matcher(spacy.nlp.vocab)
+		merge_trp.matcher.add("amod", [[{"POS":"ADJ", "DEP":"amod"}, {"POS":"NOUN"}]])
+		merge_trp.matcher.add("det", [[{"POS":"DET", "DEP":"det"}, {"POS":"NOUN"}]])
+		merge_trp.matcher.add("advmod", [[{"POS":"ADV", "DEP":"advmod"}, {"POS": {"IN": ["VERB", "ADJ"]}} ]])
+		
+	with doc.retokenize() as retokenizer:
+		for name, start, end in merge_trp.matcher(doc):
+			try: 
+				attrs = {"pos": doc[start+1].pos, "tag": doc[start+1].tag, "dep": doc[start+1].dep, 
+					"lemma":doc[start+1].lemma, 
+					"ent_type": "merge_trp:" + spacy.nlp.vocab[name].text + f":{doc[start].tag_} {doc[start+1].tag_}" }
+				retokenizer.merge(doc[start : end], attrs=attrs)
+			except Exception as e:
+				print ( "merge_trp ex:", e , start, end)
+	return doc
+
+def ske(doc, name:str='sm'): # added 2023.1.25
+	if not hasattr(ske, 'nlp'):
+		ske.nlp = spacy.load(f'en_core_web_{name}')
+		[ ske.nlp.add_pipe(p) for p in ("merge_entities", "merge_np", "merge_prt", "merge_trp", "span_clause") ]
+	return ske.nlp(doc) 
+#print( ske("The quick fox jumped over the lazy dog.")[0] )
+
+def merge_span(doc, pat:list=[{"POS":"VERB"},{"LEMMA":"to"},{"TAG":"VB"}]): 
+	''' decide to save => one, 2023.1.23 ''' 
+	spat = json.dumps(pat) 
+	if not hasattr(merge_span, spat):
+		_matcher = Matcher(spacy.nlp.vocab)
+		_matcher.add(spat, [pat], greedy ='LONGEST')
+		setattr(merge_span, spat, _matcher) 
+	with doc.retokenize() as retokenizer:
+		for name, start, end in getattr(merge_span, spat)(doc):
+			try: 
+				attrs = {"pos": doc[start].pos, "tag": doc[start].tag, "dep": doc[start].dep, 
+					"lemma":"_".join([doc[i].lemma_ if i == start else doc[i].text for i in range(start,end)]), 
+					"ent_type": "merge_span:" + spat }
+				retokenizer.merge(doc[start : end], attrs=attrs)
+			except Exception as e:
+				print ( "merge_v ex:", e , start, end)
 	return doc
 
 def depmatch(): #from spacy.matcher import Matcher,DependencyMatcher
@@ -813,6 +673,30 @@ def sqlsi(infile, func:str='toktrp', outfile:str=None,batch=1000,): #ibeg:int=0,
 	conn.commit()
 	print("sqlsi finished:", infile)
 
+def fasttext(infile, outfile:str=None,batch=1000,): 
+	''' prepare data for fasttext input, | ./fasttext skipgram -input dic.fasttext -output dic  |2023.1.26 '''
+	if outfile is None or not outfile: outfile = infile.split('.')[0] + f".fasttext"
+	print(f"started: [batch={batch}]", infile, outfile , flush=True)
+
+	def fparse(doc):
+		merge_ent(doc)
+		#merge_np(doc)  # turn_off the radio 
+		merge_v(doc) 
+		return " ".join([ f"_{t.pos_}" if t.pos_ in ("PROPN","NUM") else f"{t.lemma_}/VERB" if t.ent_type_.startswith("merge_v:") else f"_{t.ent_type_}" if t.ent_type_ else f"{t.lemma_}/{t.pos_}" if t.pos_ in ("VERB","NOUN", "ADJ","ADV") else t.text for t in doc])
+
+	start = time.time()
+	with open(outfile, 'w') as fw:
+		for sid, line in enumerate(fileinput.input(infile,openhook=fileinput.hook_compressed)): 
+			try: 
+				arr = json.loads(line.strip()) 
+				doc = spacy.from_json(arr)  
+				fw.write( fparse(doc) + "\n") 
+			except Exception as e:
+				print ("ex:", e, sid, line) 
+				exc_type, exc_value, exc_obj = sys.exc_info() 	
+				traceback.print_tb(exc_obj)
+	print("fasttext finished:", infile)
+
 def test(): 
 	doc = spacy.nlp("It is based on the book.") # "While I was thrilled that it was ok, I worried that she is happy."
 	for name, ar in depmatch()(doc) : 
@@ -837,7 +721,9 @@ def es_submit(infile, index:str=None, batch=200000, recreate:bool=True, host='17
 		try:
 			doc = Doc(spacy.nlp.vocab).from_json(json.loads(line.strip())) # add skenp 
 			for t in doc: 	add({"type":"tok", "id": f"{sid}-tok-{t.i}", "sid":sid, 'lex':t.text, 'lem':t.lemma_, 'pos':t.pos_, 'tag':t.tag_, 'dep':t.dep_, "gpos":t.head.pos_, "glem":t.head.lemma_ , "gtag":t.head.tag_ }) #'i':t.i, "head":t.head.i,
-			for sp in doc.noun_chunks: 	add({"type":"np", "id": f"{sid}-NP-{sp.start}", "sid":sid, 'chunk':sp.text, 'lem':doc[sp.end-1].lemma_ })
+			for sp in doc.noun_chunks: 	
+				if sp.end - sp.start > 1: # skip PROPN ? 
+					add({"type":"np", "id": f"{sid}-NP-{sp.start}", "sid":sid, 'chunk':sp.text, 'lem':doc[sp.end-1].lemma_ })
 			for lem, pos, type, chunk in kp_matcher(doc): #brink:NOUN:pp:on the brink # [('pp', 'on the brink', 2, 5), ('ap', 'very happy', 9, 11)]
 				add({"type":type, "id": f"{sid}-{type}-{chunk}", "sid":sid,  'chunk':chunk, 'lem':lem , "pos":pos}) #"src": doc.text,
 			for name, ar in depmatch()(doc) : 
@@ -850,7 +736,7 @@ def es_submit(infile, index:str=None, batch=200000, recreate:bool=True, host='17
 				helpers.bulk(client=es,actions=actions, raise_on_error=False)
 				print ( sid, actions[-1], flush=True)
 				actions = []
-				refresh() # 2022.1.20 
+				#refresh() # 2022.1.23 
 		except Exception as e:
 			print("ex:", e, sid)	
 			exc_type, exc_value, exc_obj = sys.exc_info() 	
@@ -862,7 +748,7 @@ def es_submit(infile, index:str=None, batch=200000, recreate:bool=True, host='17
 if __name__	== '__main__': 
 	#es_submit("gzjc.jsonlg.3.4.1.gz", host='hw13.jukuu.com')
 	import fire
-	fire.Fire({"sqlsi":sqlsi, "test":test, "es_submit":es_submit })
+	fire.Fire({"sqlsi":sqlsi, "test":test, "es_submit":es_submit, "fasttext":fasttext })
 
 def kps(doc): 
 	''' doc is of single sent, 2022.9.8 '''
@@ -897,6 +783,26 @@ def kps(doc):
 			_kps.append(f"{name}:{doc[ibeg].pos_}_{doc[ibeg].lemma_}:{doc[ibeg].lemma_} {doc[ibeg+1].lemma_} {doc[ibeg+2].text}")
 	return _kps
 
+
+def to_docbin(doc):
+	doc_bin= spacy.tokens.DocBin()
+	doc_bin.add(doc)
+	return doc_bin.to_bytes()
+
+def from_docbin(bs): 
+	return list(spacy.tokens.DocBin().from_bytes(bs).get_docs(spacy.nlp.vocab))[0] if bs else None
+
+def model(name:str='sm'):
+	''' sm/md/lg, doc = en.model('lg')("hello world"),  2023.1.17 '''
+	if not hasattr(model, name):  
+		setattr(model, name, spacy.load(f'en_core_web_{name}') )
+	return getattr(model, name) 
+lookup	= lambda s, name='lg': model(name).vocab[s].text
+fromjson= lambda jsons, name='lg': Doc(model(name).vocab).from_json(jsons)
+
+def getdoc(snt, bs):  # execute("GET", f"bytes:{snt}") 
+	return Doc(spacy.nlp.vocab).from_bytes(bs) if bs else spacy.nlp(snt) 
+
 #print(sntbr("[\u8bd1\u6587]The 55-kilometre Hong Kong Zhuhai-Macau Bridge is an extraordinary engineering. It is the world's longest sea-crossing transportation system combining bridges and tunnels, which joins the three cities of Hong Kong Zhuhai and Macao, cutting the travelling time among them from 3 hours to 30 minutes. The reinforced concrete bridge with huge spans fully not only proves that China has the ability to complete the record-breaking mega-construction, but also will enhance the regional integration and boost the economic growth. It plays a crucial role in the overall plan to develop China’s Great Bay Area, which China intends to turn into one rivaling those of San Francisco, New York and Tokyo in terms of technological innovation and economic prosperity.", with_pid=True))
 #c:\users\zhang\appdata\local\programs\python\python38\lib\site-packages  
 #/home/ubuntu/.local/lib/python3.8/site-packages/en
