@@ -18,7 +18,7 @@ def cursor_rows(query="select dep, gov, lem, pos, count(*) cnt from gzjc where t
 	cursor=''
 	while True : 
 		res = cursor_sql(query, cursor)  
-		rows.extend(res['rows'])
+		if 'rows' in res: rows.extend(res['rows'])
 		cursor = res.get('cursor','') 
 		if not cursor: break
 	return rows 
