@@ -2,6 +2,22 @@
 from uvirun import * 
 import json, requests,time,sys, traceback, difflib,os, en
 
+def sntbr(essay, trim:bool=False, with_pid:bool=False): 
+	''' sntbr("[\u8bd1\u6587]The 55-kilometre Hong Kong Zhuhai-Macau Bridge is an extraordinary engineering. It is the world's longest sea-crossing transportation system combining bridges and tunnels, which joins the three cities of Hong Kong Zhuhai and Macao, cutting the travelling time among them from 3 hours to 30 minutes. The reinforced concrete bridge with huge spans fully not only proves that China has the ability to complete the record-breaking mega-construction, but also will enhance the regional integration and boost the economic growth. It plays a crucial role in the overall plan to develop China’s Great Bay Area, which China intends to turn into one rivaling those of San Francisco, New York and Tokyo in terms of technological innovation and economic prosperity.") '''
+	from spacy.lang import en
+	if not hasattr(sntbr, 'inst'): 
+		sntbr.inst = en.English()
+		sntbr.inst.add_pipe("sentencizer")
+
+	doc = sntbr.inst(essay)
+	if not with_pid: return [ snt.text.strip() if trim else snt.text for snt in  doc.sents]
+	pid = 0 #spacy.sntpidoff	= lambda essay: (pid:=0, doc:=spacy.sntbr(essay), [ ( pid := pid + 1 if "\n" in snt.text else pid,  (snt.text, pid, doc[snt.start].idx))[-1] for snt in  doc.sents] )[-1]
+	arr = []
+	for snt in  doc.sents:
+		if "\n" in snt.text: pid = pid + 1 
+		arr.append( (snt.text, pid) ) 
+	return arr 
+
 trans_diff		= lambda src, trg:  [] if src == trg else [s for s in difflib.ndiff(src, trg) if not s.startswith('?')] #src:list, trg:list
 trans_diff_merge= lambda src, trg:  [] if src == trg else [s.strip() for s in "^".join([s for s in difflib.ndiff(src, trg) if not s.startswith('?')]).replace("^+","|+").split("^") if not s.startswith("+") ]
 def mkf_input(snts, docs, tokenizer, sntdic:dict={},diffmerge:bool=False): 
@@ -59,7 +75,7 @@ def xgecmkf(formula:dict={
 				"ttr1":[3.43, 4.28, 5.2, 6, 6.8,					3, 0.0882, 0.5],
 				"word_diff_avg":[4.47, 4.73, 5.25,5.8, 6.6,			1, 0.0441, 0.0967],
 				"word_gt7":[0.11, 0.19, 0.3, 0.42, 0.49,			1, 0.0588, 0.1289]}, 
-				essay_or_snts:str="She has ready. It are ok.", timeout:int=5, ttl:int=87200, gechost:str="192.168.201.120:6379",dskhost:str="172.17.0.1:7095"): # rgec_host:str='192.168.201.120', rgec_port:int=6379 , dskhost:str='172.17.0.1:7095'
+				essay_or_snts:str="She has ready. It are ok.", timeout:int=5, ttl:int=87200,  gechost:str="192.168.201.120:6379",dskhost:str="172.17.0.1:7095"): # rgec_host:str='192.168.201.120', rgec_port:int=6379 , dskhost:str='172.17.0.1:7095'
 	''' 智慧课堂， 大量重复作文提交，1. with cache 2, gechost in the whiteIP list, 2022.10.30 '''
 	import redis
 	from dsk import score, sntsdims 
@@ -94,6 +110,30 @@ def xgecmkf(formula:dict={
 		"dims": dims, "score": dscore }
 	#return [ {"snt": snt, "spacy":spacy, "mkf":mkf} for snt,spacy, mkf in zip( snts,  xgecmkf.r.mget([f"spacy:{snt}" for snt in snts]),  xgecmkf.r.mget([f"mkf:{snt}" for snt in snts]) ) ]
 
+@app.get('/xdsk_with_pid', tags=["dsk"])
+def xdsk_with_pid(essay_or_snts:str="She has ready. It are ok.", timeout:int=5): 
+	''' 学科网需要段落信息， 2022.11.12 '''
+	import redis
+	if not hasattr(xdsk_with_pid,'r'): xdsk_with_pid.r = redis.Redis(host=gec_redis.split(":")[0], port=int(gec_redis.split(":")[-1]), decode_responses=True)
+
+	snts_pids	= sntbr(essay_or_snts, with_pid=True)
+	snts		= [snt for snt, pid in snts_pids] 
+	def	gec(snts): 
+		if not snts: return {}
+		id		= xdsk_with_pid.r.xadd("xsnts", {'snts':json.dumps(snts)})
+		res		= xdsk_with_pid.r.blpop([f"suc:{id}",f"err:{id}"], timeout=timeout)
+		xdsk_with_pid.r.xdel("xsnts", id)
+		return	{} if res is None else json.loads(res[1])
+	sntdic	= gec(snts) 
+
+	docs	= [spacy.nlp(snt) for snt in snts ] 
+	input	= mkf_input(snts, docs, spacy.nlp.tokenizer, sntdic)
+	dsk		= requests.post(f"http://{dsk_host}/parser", data={"q":json.dumps({"snts":input, "rid":"10"}).encode("utf-8")}).json()
+	if len(snts_pids) == len(dsk.get('snt',[]) ) : 
+		for i, mkf in enumerate(dsk.get('snt',[])): 
+			mkf['meta'].update({'pid': snts_pids[i][1]})
+	return  dsk
+
 @app.get('/dsk/register_redis', tags=["dsk"])
 def register_redis(rinfo:str='172.17.0.1:6379:0'):
 	''' to enable redis cache for snt parsing , 2022.6.30 '''
@@ -267,25 +307,9 @@ def dsk_xessay(arr:dict={"essay_or_snts":"She has ready. It are ok."}, dskhost:s
 	return {"hdoc": arr, "hsnts": sntdic} 
 
 if __name__ == '__main__':  #uvicorn.run(app, host='0.0.0.0', port=80)
-	print ( xgecmkf()) 
 	#print (dsk_wrapper("[\u8bd1\u6587]The 55-kilometre Hong Kong Zhuhai-Macau Bridge is an extraordinary engineering. It is the world's longest sea-crossing transportation system combining bridges and tunnels, which joins the three cities of Hong Kong Zhuhai and Macao, cutting the travelling time among them from 3 hours to 30 minutes. The reinforced concrete bridge with huge spans fully not only proves that China has the ability to complete the record-breaking mega-construction, but also will enhance the regional integration and boost the economic growth. It plays a crucial role in the overall plan to develop China’s Great Bay Area, which China intends to turn into one rivaling those of San Francisco, New York and Tokyo in terms of technological innovation and economic prosperity.", JSONEachRow=True))
 	#print (sntbr("[\u8bd1\u6587]The 55-kilometre Hong Kong Zhuhai-Macau Bridge is an extraordinary engineering. It is the world's longest sea-crossing transportation system combining bridges and tunnels, which joins the three cities of Hong Kong Zhuhai and Macao, cutting the travelling time among them from 3 hours to 30 minutes. The reinforced concrete bridge with huge spans fully not only proves that China has the ability to complete the record-breaking mega-construction, but also will enhance the regional integration and boost the economic growth. It plays a crucial role in the overall plan to develop China’s Great Bay Area, which China intends to turn into one rivaling those of San Francisco, New York and Tokyo in terms of technological innovation and economic prosperity."))
-
-def sntbr(essay, trim:bool=False, with_pid:bool=False): 
-	''' sntbr("[\u8bd1\u6587]The 55-kilometre Hong Kong Zhuhai-Macau Bridge is an extraordinary engineering. It is the world's longest sea-crossing transportation system combining bridges and tunnels, which joins the three cities of Hong Kong Zhuhai and Macao, cutting the travelling time among them from 3 hours to 30 minutes. The reinforced concrete bridge with huge spans fully not only proves that China has the ability to complete the record-breaking mega-construction, but also will enhance the regional integration and boost the economic growth. It plays a crucial role in the overall plan to develop China’s Great Bay Area, which China intends to turn into one rivaling those of San Francisco, New York and Tokyo in terms of technological innovation and economic prosperity.") '''
-	from spacy.lang import en
-	if not hasattr(sntbr, 'inst'): 
-		sntbr.inst = en.English()
-		sntbr.inst.add_pipe("sentencizer")
-
-	doc = sntbr.inst(essay)
-	if not with_pid: return [ snt.text.strip() if trim else snt.text for snt in  doc.sents]
-	pid = 0 #spacy.sntpidoff	= lambda essay: (pid:=0, doc:=spacy.sntbr(essay), [ ( pid := pid + 1 if "\n" in snt.text else pid,  (snt.text, pid, doc[snt.start].idx))[-1] for snt in  doc.sents] )[-1]
-	arr = []
-	for snt in  doc.sents:
-		if "\n" in snt.text: pid = pid + 1 
-		arr.append( (snt.text, pid) ) 
-	return arr 
+	print ( xdsk_with_pid()) 
 
 def topk_info(snts, docs,  topk, default_dims = {"internal_sim":0.2} ): 
 	''' info computing with topk snts, upon those long essay '''
