@@ -40,6 +40,25 @@ def gramx_single_post(arr:dict={"body": "Parents _3_ much importance to educatio
 def gramx_grams(grams: str = 'overcome difficulty,overcame difficulty,overcame difficulty***', sepa:str=','): 
 	return { gram.strip(): gramx(gram.strip()) for gram in grams.strip().split(sepa) }
 
+from itertools import product
+def decouple(query = "receive/accept the/a/ education"):
+	''' at least two items '''
+	rows = [ q.split('/') for q in query.strip().split(' ')] #[['receive', 'accept'], ['the', 'a', 'an'], ['education']]
+	res = [' '.join(row) for row in product(rows[0], rows[1])]
+	for i in range(2, len(rows)): 
+		res = [' '.join(row) for row in product(res, rows[i])]
+	return [row.replace('  ',' ') for row in res] # use re, to replace multiple spaces 
+
+@app.get('/gramx/query', tags=["gramx"])
+def gramx_query(q: str = 'receive/accept the/a/ education'): 
+	''' imple of linggle, 2022.12.12 '''
+	rows = decouple(q) 
+	res	 = gramx_grams( "|".join(rows), "|")
+	sumi = sum([i for s,i in res.items()]) + 0.001 
+	data = [ {"gram": k, "count": v, "perc": round(100 * v / sumi, 1)} for k,v in res.items()]
+	data.sort(key=lambda a:a['count'], reverse=True)
+	return {"query": q, "rows": rows, "gramcnt": res, "data": data }
+
 @app.get('/gramx/startswith', tags=["gramx"])
 def gramx_startswith(prefix: str = 'overcome diffi', topk:int=10): 
 	i = prefix.strip().count(' ') + 1
